--[[
	Cooldown Timeline, Vreenak (US-Remulos)
	https://www.curseforge.com/wow/addons/cooldown-timeline
]]--

function CooldownTimeline:CreateBarFrame()
	-- Create the fast lane frame
	CooldownTimeline.fBar = CreateFrame("Frame", "CooldownTimeline_Bar", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	-- Create the other parts
	self.fBar.bg = self.fBar:CreateTexture(nil, "BACKGROUND")
	--self.fBar.border = CreateFrame("Frame", "CooldownTimeline_Timeline_Border", CooldownTimeline_Timeline, BackdropTemplateMixin and "BackdropTemplate" or nil)
	--self.fBar.agIN, self.fBar.agOUT = CooldownTimeline:SetTimelineAnimations()
	
	-- Set things up 
	CooldownTimeline:SetBarFrame()
	
	-- OnUpdate
	self.fBar:HookScript("OnUpdate", function(self, elapsed)
		BarUpdate(self, elapsed)
	end)
	
	-- Drag and drop movement
	self.fBar:RegisterForDrag("LeftButton")
	self.fBar:SetScript("OnDragStart", self.fBar.StartMoving)
	self.fBar:SetScript("OnDragStop", self.fBar.StopMovingOrSizing)
	self.fBar.unlockTexture = self.fBar:CreateTexture(nil, "OVERLAY")
	self.fBar.unlockTexture:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", "Flat"))
	self.fBar.unlockTexture:SetAllPoints()
	self.fBar.unlockTexture:SetColorTexture(0.15, 0.15, 0.15, 1)
	self.fBar.unlockText = self.fBar:CreateFontString(nil, "OVERLAY")
	self.fBar.unlockText:SetFont("Fonts\\ARIALN.ttf", 12, "OUTLINE")
	self.fBar.unlockText:SetPoint("CENTER", 0, 0)
	self.fBar.unlockText:SetText("BARS\nDrag to Move")
	if not CooldownTimeline.db.profile.unlockFrames then
		self.fBar.unlockTexture:Hide()
		self.fBar.unlockText:Hide()
	end
	
	-- Debug text
	self.fBar.text = self.fBar:CreateFontString(nil,"ARTWORK")
	self.fBar.text:SetFont("Fonts\\ARIALN.ttf", 12, "OUTLINE")
	self.fBar.text:SetPoint("TOPLEFT",0,15)
	self.fBar.text:SetText("*** Bars ***")
	
	if not self.db.profile.debugFrame then
		self.fBar.text:Hide()
	end
end

-- Things we need to do to refresh the timeline
function CooldownTimeline:SetBarFrame()
	-- Timeline settings
	local fBarFramePosX = self.db.profile.fBarFramePosX
	local fBarFramePosY = self.db.profile.fBarFramePosY
	local fBarFrameRelativeTo = self.db.profile.fBarFrameRelativeTo
	
	-- Set the timeline
	self.fBar:SetPoint(fBarFrameRelativeTo, fBarFramePosX, fBarFramePosY)
	self.fBar:SetWidth(self.db.profile.fBarFrameWidth)
	self.fBar:SetHeight(self.db.profile.fBarFrameHeight)
	
	-- Set the timeline background
	self.fBar.bg:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", self.db.profile.fBarFrameBackground))
	self.fBar.bg:SetAllPoints(true)
	self.fBar.bg:SetVertexColor(
		CooldownTimeline.db.profile.fBarFrameBackgroundColor["r"],
		CooldownTimeline.db.profile.fBarFrameBackgroundColor["g"],
		CooldownTimeline.db.profile.fBarFrameBackgroundColor["b"],
		CooldownTimeline.db.profile.fBarFrameBackgroundColor["a"]
	)
		
	-- Border settings
	--[[local fTimelineBorder = CooldownTimeline.db.profile.fTimelineBorder
	local fTimelineBorderSize = CooldownTimeline.db.profile.fTimelineBorderSize
	local fTimelineBorderInset = CooldownTimeline.db.profile.fTimelineBorderInset
	local fTimelineBorderPadding = CooldownTimeline.db.profile.fTimelineBorderPadding
	local fTimelineBorderColor = CooldownTimeline.db.profile.fTimelineBorderColor
	
	-- Set the border
	CooldownTimeline:SetBorder(self.fBar, fTimelineBorder, fTimelineBorderSize, fTimelineBorderInset)
	CooldownTimeline:SetBorderColor(self.fBar, fTimelineBorderColor)
	CooldownTimeline:SetBorderPoint(self.fBar, fTimelineBorderPadding)
	self.fBar.border:SetFrameLevel(CooldownTimeline_Timeline:GetFrameLevel() + 1)]]--
end

function CooldownTimeline:CreateCooldownBar(data)
	local name = "CooldownTimeline_Bar_"..tostring(data["name"])
	local f = CreateFrame("StatusBar", name, CooldownTimeline_Bar, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	f.cdUniqueID = CooldownTimeline:AssignUniqueID()
	f.cdName = data["name"]
	f.cdType = data["type"]
	f.cdID = data["id"]
	
	-- Create the required parts
	f.bg = f:CreateTexture(nil, "BACKGROUND")
	--local tiName = "CooldownTimeline_Bar_"..f.cdName
	
	f.ti = CreateFrame("Frame", name.."TI", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.ti:SetParent(name)
	--f.ti:SetFrameLevel(20)
	
	
	f.text = CreateFrame("Frame", name.."Text", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	f.text:SetParent(name)
	--f.text:SetFrameLevel(30)
	
	f.ti.bg = f.ti:CreateTexture(nil, "BACKGROUND")
	f.text.text1 = f.text:CreateFontString(nil, "ARTWORK")
	f.text.text2 = f.text:CreateFontString(nil, "ARTWORK")
	f.text.text3 = f.text:CreateFontString(nil, "ARTWORK")
	--f.text.rightText = f.text:CreateFontString(nil, "ARTWORK")
	
	-- Set the timeline
	CooldownTimeline:SetCooldownBar(f)
	
	-- OnUpdate
	f:HookScript("OnUpdate", function(self, elapsed)
		CooldownBarUpdate(self, elapsed)
	end)
	
	-- Show the icon unique ID if we are in debug mode
	f.textID = f:CreateFontString(nil,"ARTWORK")
	f.textID:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	f.textID:SetPoint("TOPLEFT",2,-2)
	f.textID:SetText(f.cdUniqueID)
	
	f.textName = f:CreateFontString(nil,"ARTWORK")
	f.textName:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	f.textName:SetPoint("BOTTOMLEFT",2,-2)
	--f.textName:SetText(f:GetName())
	f.textName:SetText(f:GetName():gsub("CooldownTimeline_", ""))
	
	if not self.db.profile.debugFrame then
		f.textID:Hide()
		f.textName:Hide()
	end
	
	-- Finally, add the icon to the icon table
	table.insert(CooldownTimeline.barTable, f)
end

function CooldownTimeline:SetCooldownBar(bar)
	local fBarWidth = self.db.profile.fBarWidth
	local fBarHeight = self.db.profile.fBarHeight
	
	local fBarTexture = self.db.profile.fBarTexture
	local fBarTextureColor = self.db.profile.fBarTextureColor
	
	local fBarBackground = self.db.profile.fBarBackground
	local fBarBackgroundColor = self.db.profile.fBarBackgroundColor
	
	local fBarFont = self.db.profile.fBarFont
	local fBarFontSize = self.db.profile.fBarFontSize
	local fBarFontColor = self.db.profile.fBarFontColor

	bar:SetPoint("CENTER", 0, 0)
	bar:SetWidth(fBarWidth)
	bar:SetHeight(fBarHeight)

	bar:SetMinMaxValues(0, 1)
	bar:SetReverseFill(CooldownTimeline.db.profile.fBarDirectionReverse)

	-- Set the bar texture
	bar:SetStatusBarTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", fBarTexture))
	bar:GetStatusBarTexture():SetHorizTile(false)
	bar:GetStatusBarTexture():SetVertTile(false)
	bar:SetStatusBarColor(
		fBarTextureColor["r"],
		fBarTextureColor["g"],
		fBarTextureColor["b"],
		fBarTextureColor["a"]
	)
	
	-- Set the background texture
	bar.bg:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", fBarBackground))
	bar.bg:SetAllPoints(true)
	bar.bg:SetVertexColor(
		fBarBackgroundColor["r"],
		fBarBackgroundColor["g"],
		fBarBackgroundColor["b"],
		fBarBackgroundColor["a"]
	)
	
	-- Set the indicator
	local fBarTransitionTextureWidth = self.db.profile.fBarTransitionTextureWidth
	local fBarTransitionTextureHeight = self.db.profile.fBarTransitionTextureHeight
	
	local fBarTransitionTexture = self.db.profile.fBarTransitionTexture
	local fBarTransitionTextureColor = self.db.profile.fBarTransitionTextureColor
	
	bar.ti:SetPoint("CENTER", 0, 0)
	bar.ti:SetWidth(fBarTransitionTextureWidth)
	bar.ti:SetHeight(fBarHeight)
	
	bar.ti.bg:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", fBarTransitionTexture))
	bar.ti.bg:SetAllPoints(true)
	bar.ti.bg:SetVertexColor(
		fBarTransitionTextureColor["r"],
		fBarTransitionTextureColor["g"],
		fBarTransitionTextureColor["b"],
		fBarTransitionTextureColor["a"]
	)
	
	-- Set the text
	bar.text:SetPoint("CENTER", 0, 0)
	bar.text:SetWidth(fBarWidth)
	bar.text:SetHeight(fBarHeight)
	
	local t1 =  self.db.profile.fBarText1
	bar.text.text1:SetFont(CooldownTimeline.SharedMedia:Fetch("font", t1["font"]), t1["size"], t1["outline"])
	bar.text.text1:ClearAllPoints()
	bar.text.text1:SetPoint(t1["align"], bar.text, t1["anchor"], t1["xOffset"], t1["yOffset"])
	bar.text.text1:SetText(CooldownTimeline:ConvertTextTags(t1["text"]))
	bar.text.text1:SetTextColor(
		t1["color"]["r"],
		t1["color"]["g"],
		t1["color"]["b"],
		t1["color"]["a"]
	)
	bar.text.text1:SetShadowColor(
		t1["shadowColor"]["r"],
		t1["shadowColor"]["g"],
		t1["shadowColor"]["b"],
		t1["shadowColor"]["a"]
	)
	bar.text.text1:SetShadowOffset(t1["shadowXOffset"], t1["shadowYOffset"])
	bar.text.text1:SetNonSpaceWrap(false)
	if not t1["enabled"] then
		bar.text.text1:Hide()
	end
	
	local t2 =  self.db.profile.fBarText2
	bar.text.text2:SetFont(CooldownTimeline.SharedMedia:Fetch("font", t2["font"]), t2["size"], t2["outline"])
	bar.text.text2:ClearAllPoints()
	bar.text.text2:SetPoint(t2["align"], bar.text, t2["anchor"], t2["xOffset"], t2["yOffset"])
	bar.text.text2:SetText(CooldownTimeline:ConvertTextTags(t2["text"]))
	bar.text.text2:SetTextColor(
		t2["color"]["r"],
		t2["color"]["g"],
		t2["color"]["b"],
		t2["color"]["a"]
	)
	bar.text.text2:SetShadowColor(
		t2["shadowColor"]["r"],
		t2["shadowColor"]["g"],
		t2["shadowColor"]["b"],
		t2["shadowColor"]["a"]
	)
	bar.text.text2:SetShadowOffset(t2["shadowXOffset"], t2["shadowYOffset"])
	bar.text.text2:SetNonSpaceWrap(false)
	if not t2["enabled"] then
		bar.text.text2:Hide()
	end
	
	local t3 =  self.db.profile.fBarText3
	bar.text.text3:SetFont(CooldownTimeline.SharedMedia:Fetch("font", t3["font"]), t3["size"], t3["outline"])
	bar.text.text3:ClearAllPoints()
	bar.text.text3:SetPoint(t3["align"], bar.text, t3["anchor"], t3["xOffset"], t3["yOffset"])
	bar.text.text3:SetText(CooldownTimeline:ConvertTextTags(t3["text"]))
	bar.text.text3:SetTextColor(
		t3["color"]["r"],
		t3["color"]["g"],
		t3["color"]["b"],
		t3["color"]["a"]
	)
	bar.text.text3:SetShadowColor(
		t3["shadowColor"]["r"],
		t3["shadowColor"]["g"],
		t3["shadowColor"]["b"],
		t3["shadowColor"]["a"]
	)
	bar.text.text3:SetShadowOffset(t3["shadowXOffset"], t3["shadowYOffset"])
	bar.text.text3:SetNonSpaceWrap(false)
	if not t3["enabled"] then
		bar.text.text3:Hide()
	end
	
	--if not self.db.profile.fBarTransitionIndicator and not self.db.profile.fBarOnlyShowOverThreshold then
		bar.ti:Hide()
	--end
end

function BarUpdate(self, elapsed)
	if CooldownTimeline.db.profile.enableBars then
		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
	
	if CooldownTimeline.db.profile.hideOutsideCombat or CooldownTimeline.db.profile.onlyShowWhenCoolingDown then
		if CooldownTimeline:ShouldHide(self) then
			-- Straight off
			self:SetAlpha(0)
		else
			-- Straight on
			self:SetAlpha(1)
		end
	end
	
	-- Deal with frame unlocking
	if CooldownTimeline.db.profile.unlockFrames then
		CooldownTimeline:UnlockFrame(self)

		local _, _, relativeTo, xOfs, yOfs = self:GetPoint()
		
		CooldownTimeline.db.profile.fBarFrameRelativeTo = relativeTo
		CooldownTimeline.db.profile.fBarFramePosX = xOfs
		CooldownTimeline.db.profile.fBarFramePosY = yOfs
	else
		CooldownTimeline:LockFrame(self)
	end

	
	
	-- Sort and position the bars as required
	local children = { CooldownTimeline.fBar:GetChildren() }
	
	local childCount = 0
	for _, child in ipairs(children) do
		local xPosition = 0
		local yPosition = CooldownTimeline.db.profile.fBarHeight * childCount
		
		local xPadding = CooldownTimeline.db.profile.fBarXPadding * childCount
		local yPadding = CooldownTimeline.db.profile.fBarYPadding * childCount
		
		child:ClearAllPoints()
		child:SetPoint("CENTER", xPosition + xPadding, -yPosition + -yPadding)
		
		childCount = childCount + 1
	end

	if CooldownTimeline.db.profile.debugFrame then
		self.text:Show()
		
		local children = { self:GetChildren() }
		
		local childCount = 0
		for _, child in ipairs(children) do
			if child.cdUniqueID then
				childCount = childCount + 1
			end
		end
		
		self.text:SetText("*** Bars ("..childCount..") ***")
	else
		self.text:Hide()
	end
end

function CooldownBarUpdate(frame, elapsed)
	if CooldownTimeline.db.profile.debugFrame then
		frame.textID:Show()
		frame.textName:Show()
	else
		frame.textID:Hide()
		frame.textName:Hide()
	end
	
	if CooldownTimeline.db.profile.fBarOnlyShowOverThreshold and CooldownTimeline.db.profile.fBarTransitionIndicator then
		frame.ti:Show()
	else
		frame.ti:Hide()
	end
end