--[[
	Cooldown Timeline, Vreenak (US-Remulos)
	https://www.curseforge.com/wow/addons/cooldown-timeline
]]--

function CooldownTimeline:CreateFastlaneFrame()
	-- Create the fast lane frame
	CooldownTimeline.fFastlane = CreateFrame("Frame", "CooldownTimeline_Fastlane", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	-- Create the other parts
	self.fFastlane.bg = self.fFastlane:CreateTexture(nil, "BACKGROUND")
	--self.fFastlane.border = CreateFrame("Frame", "CooldownTimeline_Timeline_Border", CooldownTimeline_Timeline, BackdropTemplateMixin and "BackdropTemplate" or nil)
	--self.fFastlane.agIN, self.fFastlane.agOUT = CooldownTimeline:SetTimelineAnimations()
	
	-- Set things up 
	CooldownTimeline:SetFastlaneFrame()
	
	-- OnUpdate
	self.fFastlane:HookScript("OnUpdate", function(self, elapsed)
		FastlaneUpdate(self, elapsed)
	end)
	
	-- Drag and drop movement
	self.fFastlane:RegisterForDrag("LeftButton")
	self.fFastlane:SetScript("OnDragStart", self.fFastlane.StartMoving)
	self.fFastlane:SetScript("OnDragStop", self.fFastlane.StopMovingOrSizing)
	self.fFastlane.unlockTexture = self.fFastlane:CreateTexture(nil, "OVERLAY")
	self.fFastlane.unlockTexture:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", "Flat"))
	self.fFastlane.unlockTexture:SetAllPoints()
	self.fFastlane.unlockTexture:SetColorTexture(0.15, 0.15, 0.15, 1)
	self.fFastlane.unlockText = self.fFastlane:CreateFontString(nil, "OVERLAY")
	self.fFastlane.unlockText:SetFont("Fonts\\ARIALN.ttf", 12, "OUTLINE")
	self.fFastlane.unlockText:SetPoint("CENTER", 0, 0)
	self.fFastlane.unlockText:SetText("FASTLANE\nDrag to Move")
	if not CooldownTimeline.db.profile.unlockFrames then
		self.fFastlane.unlockTexture:Hide()
		self.fFastlane.unlockText:Hide()
	end
	
	-- Debug text
	self.fFastlane.text = self.fFastlane:CreateFontString(nil,"ARTWORK")
	self.fFastlane.text:SetFont("Fonts\\ARIALN.ttf", 12, "OUTLINE")
	self.fFastlane.text:SetPoint("TOPLEFT",0,15)
	self.fFastlane.text:SetText("*** Fastlane ***")
	
	if not self.db.profile.debugFrame then
		self.fFastlane.text:Hide()
	end
	
	--CooldownTimeline:Print("Created Fast Lane")
end

-- Things we need to do to refresh the timeline
function CooldownTimeline:SetFastlaneFrame()
	-- Timeline settings
	local fFastlaneRelativeTo = self.db.profile.fFastlaneRelativeTo
	local fFastlanePosX = self.db.profile.fFastlanePosX
	local fFastlanePosY = self.db.profile.fFastlanePosY
	local fFastlaneWidth = self.db.profile.fFastlaneWidth
	local fFastlaneHeight = self.db.profile.fFastlaneHeight
	
	local fFastlaneBackground = self.db.profile.fFastlaneBackground
	local fFastlaneBackgroundColor = self.db.profile.fFastlaneBackgroundColor
	
	-- Set the timeline
	self.fFastlane:SetPoint(fFastlaneRelativeTo, fFastlanePosX, fFastlanePosY)
	self.fFastlane:SetWidth(fFastlaneWidth)
	self.fFastlane:SetHeight(fFastlaneHeight)
	
	-- Set the timeline background
	self.fFastlane.bg:SetTexture(CooldownTimeline.SharedMedia:Fetch("statusbar", fFastlaneBackground))
	self.fFastlane.bg:SetAllPoints(true)
	self.fFastlane.bg:SetVertexColor(
		fFastlaneBackgroundColor["r"],
		fFastlaneBackgroundColor["g"],
		fFastlaneBackgroundColor["b"],
		fFastlaneBackgroundColor["a"]
	)
	
	--self.fFastlane:SetAlpha(1)
	
	-- Border settings
	--[[local fTimelineBorder = CooldownTimeline.db.profile.fTimelineBorder
	local fTimelineBorderSize = CooldownTimeline.db.profile.fTimelineBorderSize
	local fTimelineBorderInset = CooldownTimeline.db.profile.fTimelineBorderInset
	local fTimelineBorderPadding = CooldownTimeline.db.profile.fTimelineBorderPadding
	local fTimelineBorderColor = CooldownTimeline.db.profile.fTimelineBorderColor
	
	-- Set the border
	CooldownTimeline:SetBorder(self.fFastlane, fTimelineBorder, fTimelineBorderSize, fTimelineBorderInset)
	CooldownTimeline:SetBorderColor(self.fFastlane, fTimelineBorderColor)
	CooldownTimeline:SetBorderPoint(self.fFastlane, fTimelineBorderPadding)
	self.fFastlane.border:SetFrameLevel(CooldownTimeline_Timeline:GetFrameLevel() + 1)]]--
end

function FastlaneUpdate(self, elapsed)
	if not CooldownTimeline.db.profile.enableFastlane then
		self:SetAlpha(0)
	else
		self:SetAlpha(1)
	end
	
	if CooldownTimeline.db.profile.hideOutsideCombat or CooldownTimeline.db.profile.onlyShowWhenCoolingDown then
		if CooldownTimeline:ShouldHide(self) then
			-- Straight off
			self:SetAlpha(0)
		else
			-- Straight on
			self:SetAlpha(1)
		end
	end
	
	-- Deal with frame unlocking
	if CooldownTimeline.db.profile.unlockFrames then
		CooldownTimeline:UnlockFrame(self)

		local _, _, relativeTo, xOfs, yOfs = self:GetPoint()
		
		CooldownTimeline.db.profile.fFastlaneRelativeTo = relativeTo
		CooldownTimeline.db.profile.fFastlanePosX = xOfs
		CooldownTimeline.db.profile.fFastlanePosY = yOfs
	else
		CooldownTimeline:LockFrame(self)
	end
	
	if CooldownTimeline.db.profile.debugFrame then
		self.text:Show()
		
		local children = { self:GetChildren() }
		
		local childCount = 0
		for _, child in ipairs(children) do
			if child.cdUniqueID then
				childCount = childCount + 1
			end
		end
		
		self.text:SetText("*** Fastlane ("..childCount..") ***")
	else
		self.text:Hide()
	end
end